#include <iostream>
#include <fstream>
#include <string.h>

using namespace std;

class Osoba{
	public:
		Osoba();
		Osoba(const char* jmeno, const char *prijmeni, int hodnoceni);
		~Osoba();
		void set(const char* jmeno, const char *prijmeni, int hodnoceni);
		void clear();

		char *jmeno, *prijmeni;
		int hodnoceni;
};
//------------------------------------------------------------------------
struct Node
{
	Osoba os;
	Node *left;
	Node *right;
};
//------------------------------------------------------------------------
class Seznam{
	public:
		Seznam();
		~Seznam();
		void insert(Osoba os);  //Vlozi osobu na konec seznamu
		void print();           //Vytiskne seznam na monitor
		void print_rev();       //Vytiskne seznam na monitor od konce
		void sortByValue();     //Seradi seznam podle hodnoceni osoby
        Node* search(Osoba os);
		void sortByName();

	private:
		Node *first, *last;
		void swap(Node *a, Node *b); //Prohodi uzly navzajem
};
//------------------------------------------------------------------------
int main()
{
    ifstream file;
    char line[256],jmeno[100],prijmeni[100];
    int hodnoceni;
    Osoba O;
    Seznam S;

    file.open ("zaznamy.txt", ifstream::in);
    file.getline(line,256);

    while (file.good()) {
        sscanf(line,"%s %s %d",jmeno, prijmeni,&hodnoceni);
        //cout << jmeno << " " << prijmeni << " " << hodnoceni << endl;

        O.set(jmeno, prijmeni,hodnoceni);
        S.insert(O);

        file.getline(line,256);
    }
    file.close();

    S.print();
    cout << endl;

    /* Serazeni a tisk soutezicich: */
    S.sortByValue();
    S.print();

    return 0;
}

//------------------------------------------------------------------------
Osoba::Osoba(){
    jmeno = prijmeni = NULL;
    hodnoceni = 0;
}
//------------------------------------------------------------------------
Osoba::Osoba(const char* _jmeno, const char *_prijmeni, int _hodnoceni){
    jmeno = prijmeni = NULL;
    set(_jmeno,_prijmeni,_hodnoceni);
}
//------------------------------------------------------------------------
Osoba::~Osoba(){
    delete jmeno;
    delete prijmeni;
}
//------------------------------------------------------------------------
void Osoba::set(const char* _jmeno, const char *_prijmeni, int _hodnoceni){
    clear();
    jmeno = new char[strlen(_jmeno)+1];
    strcpy(jmeno,_jmeno);
    prijmeni = new char[strlen(_prijmeni)+1];
    strcpy(prijmeni,_prijmeni);
    hodnoceni = _hodnoceni;
}
//------------------------------------------------------------------------
void Osoba::clear(){
    if(jmeno != NULL) delete[] jmeno;
    if(prijmeni != NULL) delete[] prijmeni;
    jmeno = prijmeni = NULL;
    hodnoceni = 0;
}
//------------------------------------------------------------------------
Seznam::Seznam(){
    first = last = NULL;
}
//------------------------------------------------------------------------
Seznam::~Seznam(){
    Node *next,*current;
    current = first;

    while(current!=NULL){
        next = current->right;
        delete current;
        current = next;
    }
}
//------------------------------------------------------------------------
void Seznam::insert(Osoba os){
    if(last==NULL){
        first = last = new Node;
        first->left = first->right = NULL;
        last->left = last->right = NULL;
    }else{
        last->right = new Node;
        last->right->left = last;
        last->right->right = NULL;
        last = last->right;
    }
    last->os.set(os.jmeno,os.prijmeni,os.hodnoceni);
}
//------------------------------------------------------------------------
void Seznam::print(){
    Node *current = first;
    while(current != NULL){
        cout << current->os.jmeno << " " << current->os.prijmeni << " " << current->os.hodnoceni << endl;
        current = current->right;
    }
}
//------------------------------------------------------------------------
void Seznam::print_rev(){
    Node *current = last;
    while(current != NULL){
        cout << current->os.jmeno << " " << current->os.prijmeni << " " << current->os.hodnoceni << endl;
        current = current->left;
    }
}
//------------------------------------------------------------------------
void Seznam::sortByValue(){
    Node *current;
    int unsorted;
    do{ // Bubble sort
        current=first;
        unsorted = 0;
        while(current!=NULL && current->right!=NULL){
            if(current->os.hodnoceni > current->right->os.hodnoceni){
                swap(current,current->right);
                //swap(current->right,current); //I s touto verzi musi fungovat spravne...
                unsorted = 1;
            }
            current = current->right;
        }
    }while(unsorted);
}
//------------------------------------------------------------------------
void Seznam::swap(Node *a, Node *b){
    Node *aL,*aR,*bL,*bR,*tmp;

    aL = a->left;
    aR = a->right;
    bL = b->left;
    bR = b->right;

    if(b == aR) aR=a;
    if(b == aL) bR=b;

    if(aL != NULL) aL->right = b;
    b->right = aR;
    if(bL != NULL) bL->right = a;
    a->right = bR;

    b->left = aL;
    if(bR != NULL) bR->left = a;
    a->left = bL;
    if(aR != NULL) aR->left = b;

    if(a==first) first = b;
    else if(b==first) first = a;
    if(a==last)  last = b;
    else if(b==last)  last = a;
}
//------------------------------------------------------------------------
//Eof.

